/*
  Copyright (C) 1999 - 2006 Florian Schintke
 
  This is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2, or (at your option) any later 
  version. 
 
  This is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
  for more details. 
 
  You should have received a copy of the GNU General Public License with 
  the c2html, java2html, pas2html or perl2html source package as the 
  file COPYING. If not, write to the Free Software Foundation, Inc., 
  59 Temple Place - Suite 330, Boston, MA 
  02111-1307, USA. 
*/

#ifndef COLOR_H
#define COLOR_H

extern char *bgcolor;
extern char *commentcolor;
extern char *stringcolor;
extern char *definelinecolor;
extern char *preprolinecolor;
extern char *keywordcolor;
extern char *labelcolor;

#endif
